/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.marshalling;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.lang.reflect.Proxy;
import java.util.HashMap;

/**
 * Reader which use the given classloader when recreating the object.
 * @author Florent Benoit
 */
public class ClassLoaderObjectInputStream extends ObjectInputStream {

    /**
     * ClassLoader to use when loading objects.
     */
    private ClassLoader classLoader = null;

    /**
     * Primitive classes.
     */
    private static final HashMap<String, Class<?>> PRIMITIVE_CLASSES = new HashMap<String, Class<?>>();

    static {
        PRIMITIVE_CLASSES.put("boolean", boolean.class);
        PRIMITIVE_CLASSES.put("byte", byte.class);
        PRIMITIVE_CLASSES.put("char", char.class);
        PRIMITIVE_CLASSES.put("double", double.class);
        PRIMITIVE_CLASSES.put("float", float.class);
        PRIMITIVE_CLASSES.put("int", int.class);
        PRIMITIVE_CLASSES.put("long", long.class);
        PRIMITIVE_CLASSES.put("short", short.class);
        PRIMITIVE_CLASSES.put("void", void.class);
    }

    /**
     * This inputstream reader needs an input stream for its constructor.
     * @param inputStream input stream containing bytes
     * @param classLoader specific classloader to use
     * @throws IOException if super class fails
     */
    public ClassLoaderObjectInputStream(final InputStream inputStream, final ClassLoader classLoader) throws IOException {
        super(inputStream);
        this.classLoader = classLoader;
    }

    /**
     * Load the local class equivalent of the specified stream class
     * description.
     * @param desc an instance of class <code>ObjectStreamClass</code>
     * @return a <code>Class</code> object corresponding to <code>desc</code>
     * @throws IOException any of the usual input/output exceptions
     * @throws ClassNotFoundException if class of a serialized object cannot be
     *         found
     */
    @Override
    protected Class<?> resolveClass(final ObjectStreamClass desc) throws IOException, ClassNotFoundException {

        // Handle primitive classes
        String className = desc.getName();
        if (PRIMITIVE_CLASSES.containsKey(className)) {
            return PRIMITIVE_CLASSES.get(className);
        }

        // Use the thread classloader
        return Class.forName(className, false, classLoader);
    }

    /**
     * Returns a proxy class that implements the interfaces named in a proxy
     * class descriptor; subclasses may implement this method to read custom
     * data from the stream along with the descriptors for dynamic proxy
     * classes, allowing them to use an alternate loading mechanism for the
     * interfaces and the proxy class.
     * @param interfacesName the list of interface names that were deserialized in
     *        the proxy class descriptor
     * @return a proxy class for the specified interfaces
     * @throws IOException any exception thrown by the underlying
     *         <code>InputStream</code>
     * @throws ClassNotFoundException if the proxy class or any of the named
     *         interfaces could not be found
     * @see java.io.ObjectOutputStream#annotateProxyClass(Class)
     * @since 1.3
     */
    @Override
    protected Class<?> resolveProxyClass(final String[] interfacesName) throws IOException, ClassNotFoundException {
        // Load the interfaces with the
        Class<?>[] interfaces = new Class[interfacesName.length];
        for (int i = 0; i < interfaces.length; i++) {
            interfaces[i] = Class.forName(interfacesName[i], false, classLoader);
        }

        // Use the thread classloader to make the proxy
        try {
            return Proxy.getProxyClass(classLoader, interfaces);
        } catch (IllegalArgumentException e) {
            throw new ClassNotFoundException(null, e);
        }

    }

}
