/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.marshalling;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * Allow to get the bytes of an object or to recreate an object from bytes.<br>
 * It also used a custom reader when recreating object as it uses the context
 * classloader in order to have an object with the right classloader.
 * @author Florent Benoit
 */
public final class Serialization {

    /**
     * Utility class, no public constructor.
     */
    private Serialization() {

    }


    /**
     * Gets an array of bytes corresponding to the given object.
     * @param object the object to serialize
     * @return an array of bytes.
     * @throws IOException if the object can't be turned into an array of bytes.
     */
    public static byte[] storeObject(final Serializable object) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos  =  null;
        try {
            oos = new ObjectOutputStream(baos);
            oos.writeObject(object);
            oos.flush();
            return baos.toByteArray();
        } finally {
            if (oos != null)  {
                oos.close();
            }
            if (baos != null) {
                baos.close();
            }
        }
    }


    /**
     * Transforms the given array of bytes into an object.
     * @param bytes given array of bytes representing the object
     * @return object which has been unmarshalled.
     * @throws IOException if marshalling fails
     * @throws ClassNotFoundException if the class required for loading the object is not found.
     */
    public static Object loadObject(final byte[] bytes)
            throws IOException, ClassNotFoundException {
       return loadObject(bytes, null);

    }

    /**
     * Transforms the given array of bytes into an object.
     * @param bytes given array of bytes representing the object
     * @param classloader the classloader to use in order to load the Object
     * @return object which has been unmarshalled.
     * @throws IOException if marshalling fails
     * @throws ClassNotFoundException if the class required for loading the object is not found.
     */
    public static Object loadObject(final byte[] bytes, final ClassLoader classloader)
            throws IOException, ClassNotFoundException {
        // Do nothing if there aren't any bytes.
        if (bytes == null) {
            return null;
        }

        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
        ClassLoaderObjectInputStream classLoaderObjectInputStream = null;
        if (classloader != null) {
            classLoaderObjectInputStream = new ClassLoaderObjectInputStream(bais, classloader);
        } else {
            classLoaderObjectInputStream = new CtxClassLoaderObjectInputStream(bais);
        }
        try {
            return classLoaderObjectInputStream.readObject();
        } finally {
            if (classLoaderObjectInputStream != null) {
                classLoaderObjectInputStream.close();
            }
            if (classLoaderObjectInputStream != null) {
                bais.close();
            }

        }

    }

    /**
     * Clone the given serializable object.
     * @param <T> the type of the object
     * @param objectToClone the object to clone
     * @return a cloned instance
     * @throws SerializationException if the clone operation can't be done.
     */
    @SuppressWarnings("unchecked")
    public static <T extends Serializable> T cloneObject(final T objectToClone) throws SerializationException {
        // get array of bytes of the object
        byte[] bytes = null;
        try {
             bytes = storeObject(objectToClone);
        } catch (IOException e) {
            throw new SerializationException("Cannot get array of bytes of the given object " + objectToClone, e);
        }

        // Create Object from the array of bytes
        Object clonedObject = null;
        try {
            clonedObject = loadObject(bytes, objectToClone.getClass().getClassLoader());
        } catch (IOException e) {
            throw new SerializationException("Cannot create object from the array of bytes of the given object "
                    + objectToClone, e);
        } catch (ClassNotFoundException e) {
            throw new SerializationException("Cannot create object from the array of bytes of the given object "
                    + objectToClone, e);
        }

        // Perform the cast
        return (T) clonedObject;
    }

}
